@@ -19,20 +19,27 @@ import copy
 import logging
 import sys
 
+from openstack import connection
 from oslo_utils import strutils
 import six
 
 from osc_lib.api import auth
 from osc_lib import exceptions
 from osc_lib import session as osc_session
+from osc_lib import version
 
+# NOTE(dtroyer): Attempt an import to detect if the SDK installed is new
+#                enough to not use Profile.  If so, use that.
+try:
+    from openstack.config import loader as config   # noqa
+    profile = None
+except ImportError:
+    from openstack import profile
 
 LOG = logging.getLogger(__name__)
 
 PLUGIN_MODULES = []
 
-USER_AGENT = 'osc-lib'
-
 
 class ClientCache(object):
     """Descriptor class for caching created client handles."""
@@ -57,11 +64,18 @@ class ClientCache(object):
 class ClientManager(object):
     """Manages access to API clients, including authentication."""
 
+    # NOTE(dtroyer): Keep around the auth required state of the _current_
+    #                command since ClientManager has no visibility to the
+    #                command itself; assume auth is not required.
+    _auth_required = False
+
     def __init__(
         self,
         cli_options=None,
         api_version=None,
         pw_func=None,
+        app_name=None,
+        app_version=None,
     ):
         """Set up a ClientManager
 
@@ -73,11 +87,17 @@ class ClientManager(object):
             Callback function for asking the user for a password.  The function
             takes an optional string for the prompt ('Password: ' on None) and
             returns a string containing the password
+        :param app_name:
+            The name of the application for passing through to the useragent
+        :param app_version:
+            The version of the application for passing through to the useragent
         """
 
         self._cli_options = cli_options
         self._api_version = api_version
         self._pw_callback = pw_func
+        self._app_name = app_name
+        self._app_version = app_version
         self.region_name = self._cli_options.region_name
         self.interface = self._cli_options.interface
 
@@ -120,6 +140,22 @@ class ClientManager(object):
         if self.cert and self._cli_options.key:
             self.cert = self.cert, self._cli_options.key
 
+        # TODO(mordred) The logic above to set all of these is duplicated in
+        # os-client-config but needs more effort to tease apart and ensure that
+        # values are being passed in. For now, let osc_lib do it and just set
+        # the values in occ.config
+        self._cli_options.config['verify'] = self.verify
+        self._cli_options.config['cacert'] = self.cacert
+        # Attack of the killer passthrough values
+        self._cli_options.config['cert'] = self._cli_options.cert
+        self._cli_options.config['key'] = self._cli_options.key
+
+        # TODO(mordred) We also don't have any support for setting or passing
+        # in api_timeout, which is set in occ defaults but we skip occ defaults
+        # so set it here by hand and later we should potentially expose this
+        # directly to osc
+        self._cli_options.config['api_timeout'] = None
+
         # Get logging from root logger
         root_logger = logging.getLogger('')
         LOG.setLevel(root_logger.getEffectiveLevel())
@@ -162,13 +198,34 @@ class ClientManager(object):
         LOG.debug('Using parameters %s',
                   strutils.mask_password(self._cli_options.auth))
         self.auth = self._cli_options.get_auth()
+
+        if self._cli_options.service_provider:
+            self.auth = auth.get_keystone2keystone_auth(
+                self.auth,
+                self._cli_options.service_provider,
+                self._cli_options.remote_project_id,
+                self._cli_options.remote_project_name,
+                self._cli_options.remote_project_domain_id,
+                self._cli_options.remote_project_domain_name
+            )
+
         self.session = osc_session.TimingSession(
             auth=self.auth,
             verify=self.verify,
             cert=self.cert,
-            user_agent=USER_AGENT,
+            app_name=self._app_name,
+            app_version=self._app_version,
+            additional_user_agent=[('osc-lib', version.version_string)],
         )
 
+        # Create a common default SDK Connection object if it is the newer
+        # non-Profile style.
+        if profile is None:
+            self.sdk_connection = connection.Connection(
+                config=self._cli_options,
+                session=self.session,
+            )
+
         self._auth_setup_completed = True
 
     def validate_scope(self):
@@ -190,6 +247,9 @@ class ClientManager(object):
     @property
     def auth_ref(self):
         """Dereference will trigger an auth if it hasn't already"""
+        if not self._auth_required:
+            # Forcibly skip auth if we know we do not need it
+            return None
         if not self._auth_ref:
             self.setup_auth()
             LOG.debug("Get auth_ref")
